package com.landray.kmss.sys.attachment.io;

import com.landray.kmss.sys.crypt.aes.AESKeySizeEnum;
import java.nio.channels.FileChannel;
import java.io.FileInputStream;
import java.io.IOException;
import com.landray.kmss.util.StringUtil;
import com.landray.kmss.util.ResourceUtil;
import com.landray.kmss.sys.crypt.sm4.SM4Crypt;
import com.landray.kmss.sys.crypt.aes.AESCrypt;
import java.io.InputStream;

public abstract class AbstractInputStream extends InputStream
{
    protected static final String attCrypMode;
    private static final String DEFAULT_AES_PASSWORD = "MpBVxcMebDIICeAIBADANBgkqhkigEAAoGBAJeAgEAAoGBAKaEnEG9w0BAQEFAASH1D75s72SCS6zCCAmIwggJeAgEAAoGBAKaEnENEGlooCdQWYtIJ1VNxtH1D75s72";
    private static final String DEFAULT_SM4_PASSWORD = "random_seed";
    protected int cryptionMode;
    protected static final byte[] markBytes;
    protected static final byte[] aesMarkBytes;
    protected static final byte[] sm4MarkBytes;
    protected AESCrypt aesCrypt;
    protected SM4Crypt sm4Crypt;
    protected byte[] additionalBytes;
    private int count;
    protected InputStream originStream;
    private int pos;
    
    static {
        final String tempString = ResourceUtil.getKmssConfigString("sys.att.encryption.mode");
        attCrypMode = (StringUtil.isNotNull(tempString) ? tempString : "0");
        markBytes = new byte[] { 45, -89, -1, 61, -57, 5, -15, 1, -126, 8, 90, 125, -79, -14, 20, 92, 29, -11, 8, -42, 2, -64, -12, 107, 3, -109, -49, -3, -118, 120, 74, -82 };
        aesMarkBytes = new byte[] { 88, -89, -1, 61, -57, 5, -15, 1, -126, 8, 90, 125, -79, -14, 20, 92, 29, -11, 8, -42, 2, -64, -12, 107, 3, -109, -49, -3, -118, 120, 74, -82 };
        sm4MarkBytes = new byte[] { 66, -89, -1, 61, -57, 5, -15, 1, -126, 8, 90, 125, -79, -14, 20, 92, 29, -11, 8, -42, 2, -64, -12, 107, 3, -109, -49, -3, -118, 120, 74, -82 };
    }
    
    public AbstractInputStream(final InputStream originStream) throws Exception {
        this(originStream, Integer.parseInt(AbstractInputStream.attCrypMode));
    }
    
    public AbstractInputStream(final InputStream originStream, final int cryptionMode) throws Exception {
        this.additionalBytes = null;
        this.count = 0;
        this.pos = 0;
        this.cryptionMode = cryptionMode;
        this.originStream = originStream;
        this.init();
        if (this.additionalBytes == null) {
            this.count = 0;
        }
        else {
            this.count = this.additionalBytes.length;
        }
    }
    
    protected abstract void init() throws Exception;
    
    protected abstract void format(final byte[] p0, final int p1, final int p2);
    
    protected abstract int format(final int p0);
    
    @Override
    public int read() throws IOException {
        if (this.pos == this.count) {
            return this.format(this.originStream.read());
        }
        return this.format(this.additionalBytes[this.pos++]);
    }
    
    @Override
    public int available() throws IOException {
        return this.originStream.available() + this.count - this.pos;
    }
    
    public BigFileInfo getFileInfo() throws IOException {
        final FileChannel fileChannel = ((FileInputStream)this.originStream).getChannel();
        long size = fileChannel.size() + this.count - this.pos;
        if (this.cryptionMode != 0) {
            size -= AbstractInputStream.markBytes.length;
        }
        final BigFileInfo bigFileInfo = new BigFileInfo();
        bigFileInfo.setFileChannel(fileChannel);
        bigFileInfo.setFileSize(size);
        return bigFileInfo;
    }
    
    @Override
    public void close() throws IOException {
        if (this.originStream != null) {
            this.originStream.close();
        }
    }
    
    @Override
    public void mark(final int readlimit) {
        this.originStream.mark(readlimit);
    }
    
    @Override
    public boolean markSupported() {
        return this.originStream.markSupported();
    }
    
    @Override
    public int read(final byte[] b) throws IOException {
        if (this.pos == this.count) {
            final int result = this.originStream.read(b);
            this.format(b, 0, result);
            return result;
        }
        return this.read(b, 0, b.length);
    }
    
    @Override
    public int read(final byte[] b, final int off, final int len) throws IOException {
        if (this.pos == this.count) {
            final int result = this.originStream.read(b, off, len);
            this.format(b, off, result);
            return result;
        }
        int len2 = this.count - this.pos;
        int len3 = 0;
        if (len2 >= len) {
            len2 = len;
        }
        else {
            len3 = len - len2;
        }
        System.arraycopy(this.additionalBytes, this.pos, b, off, len2);
        this.pos += len2;
        if (len3 > 0) {
            if (this.originStream.available() == 0) {
                len3 = 0;
            }
            else {
                len3 = this.originStream.read(b, off + len2, len3);
            }
        }
        final int result2 = len2 + len3;
        this.format(b, off, result2);
        return result2;
    }
    
    @Override
    public void reset() throws IOException {
        this.originStream.reset();
    }
    
    @Override
    public long skip(final long n) throws IOException {
        if (this.pos == this.count) {
            return this.originStream.skip(n);
        }
        long n2 = this.count - this.pos;
        long n3 = 0L;
        if (n2 >= n) {
            n2 = n;
        }
        else {
            n3 = n - n2;
        }
        this.pos += (int)n2;
        if (n3 > 0L) {
            n3 = this.originStream.skip(n3);
        }
        return n2 + n3;
    }
    
    protected void initAES() throws Exception {
        if (this.aesCrypt != null) {
            return;
        }
        final String aesPassword = this.getAesPassword();
        this.aesCrypt = new AESCrypt(aesPassword, AESKeySizeEnum.AES_128, "AES/CFB/NoPadding");
    }
    
    private String getAesPassword() throws Exception {
        String pass = ResourceUtil.getKmssConfigString("sys.att.encryption.aespass");
        if (StringUtil.isNull(pass)) {
            pass = "MpBVxcMebDIICeAIBADANBgkqhkigEAAoGBAJeAgEAAoGBAKaEnEG9w0BAQEFAASH1D75s72SCS6zCCAmIwggJeAgEAAoGBAKaEnENEGlooCdQWYtIJ1VNxtH1D75s72";
        }
        return pass;
    }
    
    protected void initSM4() throws Exception {
        if (this.sm4Crypt != null) {
            return;
        }
        final String sm4Password = this.getSM4Password();
        this.sm4Crypt = new SM4Crypt(sm4Password);
    }
    
    private String getSM4Password() throws Exception {
        String pass = ResourceUtil.getKmssConfigString("sys.att.encryption.sm4pass");
        if (StringUtil.isNull(pass)) {
            pass = "random_seed";
        }
        return pass;
    }
    
    protected boolean isAESEncryption(final byte[] b) {
        for (int i = 0, len = b.length; i < len; ++i) {
            final byte deByte = this.aesCrypt.decrypt(b, i, 1)[0];
            if (deByte != AbstractInputStream.aesMarkBytes[i]) {
                return false;
            }
        }
        return true;
    }
    
    protected boolean isSM4Encryption(final byte[] b) {
        for (int i = 0, len = b.length; i < len; ++i) {
            final byte[] debts = this.sm4Crypt.decrypt(b, i, 1);
            if (debts == null) {
                return false;
            }
            final byte deByte = debts[0];
            if (deByte != AbstractInputStream.sm4MarkBytes[i]) {
                return false;
            }
        }
        return true;
    }
    
    private static AbstractInputStream createInputStream(final InputStream originStream, final int cryptionMode, final String cryptionMethod) throws Exception {
        AbstractInputStream result = null;
        if ("encrypt".equals(cryptionMethod)) {
            result = new EncryptionInputStream(originStream, cryptionMode);
        }
        if ("decrypt".equals(cryptionMethod)) {
            result = new DecryptionInputStream(originStream, cryptionMode);
        }
        return result;
    }
    
    public static AbstractInputStream createDecryptStream(final InputStream originStream, final int cryptionMode) throws Exception {
        return createInputStream(originStream, cryptionMode, "decrypt");
    }
    
    public static AbstractInputStream createEncryptStream(final InputStream originStream, final int cryptionMode) throws Exception {
        return createInputStream(originStream, cryptionMode, "encrypt");
    }
}
